filePollInterval = 1
fileRetryInterval = .1

from homealone import *
import json
import os
import threading
import time

class FileInterface(Interface):
    def __init__(self, name, interface=None, event=None, start=False,
                 fileName="", readOnly=False, shared=False, changeMonitor=True,
                 defaultValue=None, initialState={}):
        Interface.__init__(self, name, interface=interface, event=event)
        self.fileName = fileName
        self.readOnly = readOnly            # file cannot be written to
        self.shared = shared                # this file may be written and read by different processes
        self.changeMonitor = changeMonitor  # watch for changes and update the cache
        self.defaultValue = defaultValue    # value to use for undefined elements
        self.initialState = initialState    # values to set if the file doesn't exist
        self.data = {}                      # cached data
        self.mtime = 0                      # last time the file was modified
        self.lock = threading.Lock()
        if start:                           # immediately start the interface
            self.start()

    def start(self):
        try:
            # if the file exists, cache the data
            debug('debugFile', self.name, "reading", self.fileName)
            with open(self.fileName) as dataFile:
                self.data = json.load(dataFile)
        except FileNotFoundError:
            # create a new file
            debug('debugFile', self.name, "creating", self.fileName)
            self.data = self.initialState
            self.writeData()
        except Exception as ex:
            log(self.name, "start exception opening", self.fileName, type(ex).__name__, str(ex))
            self.data = self.initialState
            self.writeData()
        self.mtime = os.stat(self.fileName).st_mtime
        if self.changeMonitor:
            # thread to periodically check for file changes and cache the data
            def readData():
                debug('debugFileThread', self.name, "readData started")
                while True:
                    debug('debugFileThread', self.name, "waiting", filePollInterval)
                    time.sleep(filePollInterval)
                    if self.modified():     # file has changed
                        self.readData()     # read new data
                        for sensor in list(self.sensors.keys()): # notify all sensors
                            if sensor in self.data:
                                self.sensors[sensor].notify(self.data[sensor])
            readStatesThread = LogThread(name="readStatesThread", target=readData)
            readStatesThread.start()

    def read(self, addr):
        if not self.changeMonitor:   # read the file every time
            self.readData()
        with self.lock:
            try:
                value = self.data[addr]
            except KeyError:
                value = self.defaultValue
        debug('debugFileData', self.name, "read", "addr", addr, "value", value)
        return value

    def write(self, addr, value):
        if not self.readOnly:
            debug('debugFileData', self.name, "write", "addr", addr, "value", value)
            with self.lock:
                self.data[addr] = value
            self.notify()
            self.writeData()

    def delete(self, addr):
        if not self.readOnly:
            with self.lock:
                del(self.data[addr])
            self.notify()
            self.writeData()

    def modified(self):
        mtime = os.stat(self.fileName).st_mtime
        if mtime > self.mtime:
            debug('debugFile', self.name, "modified", mtime, "last", self.mtime)
            self.mtime = mtime
            return True
        else:
            return False

    def readData(self):
        try:
            with open(self.fileName) as dataFile:
                jsonData = dataFile.read()
                if (jsonData == "") and (self.shared): # wait until there is valid data
                    debug('debugFile', self.name, "readData", "waiting for data", time.time())
                    while jsonData == "":
                        time.sleep(fileRetryInterval)
                        jsonData = dataFile.read()
                    debug('debugFile', self.name, "readData", "data acquired", time.time())
            with self.lock:
                self.data = json.loads(jsonData)
        except Exception as ex:
            log(self.name, self.fileName, "readData file read error", type(ex).__name__, str(ex), "jsonData", str(jsonData))
        debug('debugFileData', self.name, "readData", self.data)

    def writeData(self):
        debug('debugFileData', self.name, "writeData", self.data)
        with open(self.fileName, "w") as dataFile:
            with self.lock:
                json.dump(self.data, dataFile)
        self.mtime = time.time()



holidayLightPatternDefault = "on"
holidayLightAnimationDefault = "solid"

import time
import threading
import random
from homealone import *

def randomPattern(length=1, colors=3):
    pattern = [0]*length
    for i in range(length):
        pattern[i] = int(16777215 * random.random()) # random number between 0 and 256**3-1
    return pattern

# define a segment of a string of lights
class Segment(object):
    def __init__(self, start, length, pattern=None, animation=None):
        self.name = "segment"
        self.start = start
        self.length = length
        if pattern:
            self.pattern = pattern
        else:
            self.pattern = [0]
        self.animation = animation
        debug("debugHolidayLights", "segment", self.name, start, length, self.pattern, self.animation)
        self.pixels = [0]*length
        self.frame = 0

    # fill the segment with a pattern
    def fill(self, pattern=None):
        if not pattern:
            pattern = self.pattern
        if isinstance(pattern[0], list):
            pattern = pattern[0]
        debug("debugHolidayLights", "segment", self.name, "fill", pattern)
        for l in range(self.length):
            self.pixels[l] = pattern[l%len(pattern)]

    # display the segment
    def display(self, strip):
        for l in range(self.length):
            strip.write(self.start+l, self.pixels[l])

    # animate the segment
    def animate(self):
        if self.animation:
            self.animation.animate(self)

    # shift the pattern by the specified number of pixels in the specified direction
    def shift(self, n, direction):
        debug("debugHolidayLightsAnimation", "segment", self.name, "shift", n, direction)
        if direction > 0:   # shift out
            pixels = self.pixels[-n:]
            self.pixels = pixels+self.pixels[:-n]
        else:       # shift in
            pixels = self.pixels[n:]
            self.pixels = pixels+self.pixels[:n]

    # turn off random pixels based on a culling frequency between 0 and 1
    def decimate(self, factor):
        debug("debugHolidayLightsAnimation", "segment", self.name, "decimate", factor)
        for p in range(len(self.pixels)):
            if random.random() < factor:
                self.pixels[p] = 0

    # dim the pattern by a specified factor
    def dim(self, factor):
        for p in range(len(self.pixels)):
            pixel = self.pixels[p]
            r = int(factor * ((pixel>>16)&0xff))
            g = int(factor * ((pixel>>8)&0xff))
            b = int(factor * (pixel&0xff))
            self.pixels[p] = (r<<16)+(g<<8)+b

# define an animation for a segment
class Animation(object):
    def __init__(self, name, rate):
        self.name = name
        self.rate = rate
        self.animationCount = 0

    # define an animation cycle
    def animate(self, segment):
        self.segment = segment
        if self.animationCount == 0:
            self.cycle()
        self.animationCount += 1
        if self.animationCount == self.rate:
            self.animationCount = 0

class RandomColorAnimation(Animation):
    def __init__(self, name="randomcolor", rate=3):
        Animation.__init__(self, name, rate)

    def cycle(self):
        self.segment.fill(randomPattern(len(self.segment.pixels)))

class CrawlAnimation(Animation):
    def __init__(self, name="crawl", rate=3, direction=1):
        Animation.__init__(self, name, rate)
        self.direction = direction

    def cycle(self):
        self.segment.shift(1, self.direction)

class SparkleAnimation(Animation):
    def __init__(self, name="sparkle", rate=3, factor=.7):
        Animation.__init__(self, name, rate)
        self.factor = factor

    def cycle(self):
        self.segment.fill()
        self.segment.decimate(self.factor)

class FlickerAnimation(Animation):
    def __init__(self, name="flicker", rate=1, factor=.7):
        Animation.__init__(self, name, rate)
        self.factor = factor

    def cycle(self):
        if random.random() < self.factor:
            self.segment.fill()
        else:
            self.segment.fill([0])

class BlinkAnimation(Animation):
    def __init__(self, name="blink", rate=15):
        Animation.__init__(self, name, rate)
        self.state = True

    def cycle(self):
        if self.state:
            self.segment.fill([off])
            self.state = False
        else:
            self.segment.fill()
            self.state = True

class FadeAnimation(Animation):
    def __init__(self, name="fade", rate=6):
        Animation.__init__(self, name, rate)
        self.fadeFactor = 10
        self.fadeIncr = -1

    def cycle(self):
        self.segment.fill()
        self.segment.dim(float(self.fadeFactor)/10)
        self.fadeFactor += self.fadeIncr
        if (self.fadeFactor == 0) or (self.fadeFactor == 10):
            self.fadeIncr = -self.fadeIncr

class MovieAnimation(Animation):
    def __init__(self, name="movie", rate=6):
        Animation.__init__(self, name, rate)

    def cycle(self):
        self.segment.fill(self.segment.pattern[self.segment.frame])
        self.segment.frame += 1
        if self.segment.frame == len(self.segment.pattern):
            self.segment.frame = 0

class HolidayLightControl(Control):
    def __init__(self, name, interface, patterns, patternControl, delay=0,
                 **kwargs):
        Control.__init__(self, name, interface, **kwargs)
        self.className = "Control"
        self.patterns = patterns                # list of patterns
        self.patternControl = patternControl    # a Control whose state specifies which pattern is active
        self.patternControl.stateSet = self.setPattern  # callback for pattern changes
        self.pattern = ""                       # name of the active pattern
        self.delay = delay                      # optional delay between animation cycles
        self.runState = off                     # current state of the control
        self.run = False                        # signal to start and stop animation thread
        self.running = False                    # animation thread is running

    def getState(self, missing=None):
        return self.runState

    def setState(self, value):
        debug("debugHolidayLights", self.name, "setState", "value:", value)
        if value:
            self.runState = on
            self.setPattern(self.patternControl, self.patternControl.getState())
        else:
           self.run = False
           self.runState = off

    # set the pattern based on the pattern control
    def setPattern(self, control, state):
        debug('debugHolidayLights', "setPattern", control.name, state)
        # stop the animation thread if it is running
        if self.running:
            self.run = False
            # wait for it to finish
            while self.running:
                time.sleep(.1)
        self.pattern = state
        self.segments = self.patterns[self.pattern]
        # compute segment positions if not explicitly specified
        pos = 0
        for segment in self.segments:
            if segment.start == 0:
                segment.start = pos
            debug("debugHolidayLights", self.name, "segment", segment.start, segment.length, segment.pattern)
            pos += segment.length
        # run the animations if the control is on
        if self.runState:
            displayThread = LogThread(name="displayThread", target=self.runDisplay)
            displayThread.start()

    # thread to run the animations
    def runDisplay(self):
        debug("debugHolidayLights", self.name, "runDisplay started")
        self.run = True
        self.running = True
        for segment in self.segments:
            segment.fill()
            segment.display(self.interface)
        self.interface.show()
        # run the animations
        while self.run:
            time.sleep(self.delay)
            for segment in self.segments:
                segment.animate()
                segment.display(self.interface)
            self.interface.show()
        # turn all lights off
        for segment in self.segments:
            segment.fill([off])
            segment.display(self.interface)
        self.interface.show()
        self.running = False
        debug("debugHolidayLights", self.name, "runDisplay terminated")




import time
import threading
from homealone import *
from homealone.resources.extraResources import *

# Compute power using a current measurement and a voltage measurement
# Voltage can either be from a specified sensor or a constant
class PowerSensor(Sensor):
    def __init__(self, name, interface=None, addr=None, currentSensor=None, voltageSensor=None, voltage=0.0, powerFactor=1.0, **kwargs):
        Sensor.__init__(self, name, interface, addr, **kwargs)
        self.className = "Sensor"
        self.currentSensor = currentSensor
        self.voltageSensor = voltageSensor
        self.voltage = voltage
        self.powerFactor = powerFactor

    def getState(self, missing=0.0):
        if self.voltageSensor:
            self.voltage = self.voltageSensor.getState(missing=0.0)
        power = self.currentSensor.getState(missing=0.0) * self.voltage * self.powerFactor
        return round(power * self.factor, self.resolution)

# Compute battery percentage using a voltage measurement
socLevels = {
            "AGM": [10.50, 11.51, 11.66, 11.81, 11.95, 12.05, 12.15, 12.30, 12.50, 12.75]
}
class BatterySensor(Sensor):
    def __init__(self, name, interface=None, addr=None, voltageSensor=None, batteryType="AGM", threshold=0.0, **kwargs):
        Sensor.__init__(self, name, interface, addr, **kwargs)
        self.className = "Sensor"
        self.voltageSensor = voltageSensor
        self.batteryType = batteryType
        self.threshold = threshold
        self.lastLevel = 0.0
        try:
            self.chargeLevels = socLevels[self.batteryType]
        except KeyError:
            self.chargeLevels = [0.0]*10

    def getState(self, missing=0.0):
        voltage = self.voltageSensor.getState(missing=0.0)

        # Use table lookup
        # level = 100
        # for chargeLevel in self.chargeLevels:
        #     if voltage < chargeLevel:
        #         level = self.chargeLevels.index(chargeLevel) * 10
        #         break

        # Calculate the percentage -  https://mycurvefit.com/
        if voltage > 13.0:
            level = 100.0
        else:
            level = 43940.64 - 11224.35*voltage + 949.6667*voltage**2 - 26.59379*voltage**3

        if level > self.threshold:
            if abs(level - self.lastLevel) > self.threshold:
                self.notify()
            self.lastLevel = level
            return level
        else:
            self.lastLevel = level
            return 0.0

# Accumulate the energy of a power measurement over time
class EnergySensor(Sensor):
    def __init__(self, name, interface=None, addr=None, powerSensor=None, poll=10, persistence=None, initial=0.0, **kwargs):
        Sensor.__init__(self, name, interface, addr, poll=poll, **kwargs)
        self.className = "Sensor"
        self.powerSensor = powerSensor
        self.persistence = persistence  # FileInterface for state persistence
        self.initial = initial
        if self.persistence:
            self.stateControl = StateControl(self.name+"State", self.persistence, self.name, initial=self.initial)
            self.energy = self.stateControl.getState(missing=0.0)
        else:
            self.energy = 0.0
        self.lastTime = time.time()

    def getState(self, missing=0.0):
        value = self.powerSensor.getState(missing=0.0)
        curTime = time.time()
        interval = curTime - self.lastTime
        self.lastTime = curTime
        self.energy += value * interval / 3600
        debug("debugEnergy", self.name, value, self.energy)
        if self.persistence:
            self.stateControl.setState(self.energy)
        return round(self.energy * self.factor, self.resolution)

    def setState(self, value):
        self.energy = value
        if self.persistence:
            self.stateControl.setState(value)

# ADS1x15 sensor
class ADSSensor(Sensor):
    def __init__(self, name, interface=None, addr=None, gain=4096, **kwargs):
        Sensor.__init__(self, name, interface, addr, **kwargs)
        self.className = "Sensor"
        self.gain = gain

    def getState(self, missing=0.0):
        self.interface.gain = self.gain
        value = round(self.interface.read(self.addr) * self.factor, self.resolution)
        debug('debugAds', self.name, "value", value)
        return value



from w1thermsensor import W1ThermSensor
from w1thermsensor import errors
from homealone import *

class W1Interface(Interface):
    def __init__(self, name, interface=None, event=None):
        Interface.__init__(self, name, interface=interface, event=event)
        self.sensors = {}
        for sensor in W1ThermSensor.get_available_sensors():
            self.sensors[sensor.id] = sensor

    def read(self, addr):
        try:
            return round(float(self.sensors[addr].get_temperature()) * 9 / 5 + 32, 2)
        except KeyError:
            return 0
        except errors.SensorNotReadyError:    # sensor isn't responding
            self.sensors = {}
            for sensor in W1ThermSensor.get_available_sensors():
                self.sensors[sensor.id] = sensor
            return 0



# Data logging, archiving, and metrics

import time
import socket
import json
import subprocess
import os
from homealone import *

class DataLogger(object):
    def __init__(self, name, appName, resources, start=False):
        self.name = name
        self.appName = appName
        self.resources = resources
        self.logDir = dataDir
        os.makedirs(self.logDir, exist_ok=True)
        archiveServers = findService(archiveService)
        if archiveServers == []:
            self.archiveServer = None
            log("no archive server found")
        else:
            self.archiveServer = archiveServers[0][0]
        if start:
            self.start()

    def start(self):
        startThread("loggingThread", self.loggingThread)

    def loggingThread(self):
        debug("debugLogging", "logging thread started")
        lastDay = ""
        while True:
            # wait for a new set of states
            states = self.resources.getStates(wait=True)
            today = time.strftime("%Y%m%d")

            # log states to a file
            if logData:
                if today != lastDay:    # start with a new baseline every day
                    lastStates = states
                changedStates = diffStates(lastStates, states, deleted=False)
                if changedStates != {}:
                    logFileName = self.logDir+today+".json"
                    debug("debugLogging", "writing states to", logFileName)
                    with open(logFileName, "a") as logFile:
                        logFile.write(json.dumps([time.time(), (changedStates if logChanged else lastStates)])+"\n")
                lastStates = states

            # send states to the metrics server
            if sendMetrics:
                debug("debugMetrics", "opening socket to", metricsHost, metricsPort)
                try:
                    metricsSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    metricsSocket.connect((metricsHost, metricsPort))
                    debug("debugMetrics", "sending", len(states), "metrics")
                    for (resourceName, state) in states.items():
                        if isinstance(state, (int, float)):     # only send numeric states
                            metricName = formatName(resourceName, ["offgrid", "solar"], ["voltage", "current", "power", "energy", "temp"])
                            msg = metricsPrefix+"."+metricName+" "+str(state)+" "+str(int(time.time()))
                            debug("debugMetricsMsg", "msg:", msg)
                            metricsSocket.send(bytes(msg+"\n", "utf-8"))
                        else:
                            debug("debugMetrics", "skipping", resourceName, state)
                except socket.error as exception:
                    log("sendMetrics", "socket error", str(exception))
                if metricsSocket:
                    debug("debugMetrics", "closing socket to", metricsHost)
                    metricsSocket.close()

            # copy to the archive server once per day
            if archiveData:
                if self.archiveServer:
                    if today != lastDay:
                        startThread("archiveDataThread", self.archiveDataThread)

            # purge logs that have been archived
            if purgeData:
                if self.archiveServer:
                    if today != lastDay:
                        startThread("purgeDataThread", self.purgeDataThread)

            lastDay = today
        debug("debugLogging", "logging thread ended")

    def archiveDataThread(self):
        debug("debugArchiveData", "archiving data logs")
        try:
            debug("debugArchiveData", "archiving "+self.logDir+" to", self.archiveServer+":"+archiveDir+self.appName+"/")
            pid = subprocess.Popen("rsync -a "+self.logDir+"* "+self.archiveServer+":"+archiveDir+self.appName+"/", shell=True)
        except Exception as ex:
            log("metrics", "exception archiving metrics", str(ex))

    def purgeDataThread(self):
        # get list of log files that are eligible to be purged
        debug("debugPurgeData", "purging logs more than", purgeDays, "days old")
        for logFile in sorted(os.listdir(self.logDir))[:-purgeDays]:
            # only purge past files
            debug("debugPurgeData", "checking", logFile)
            if logFile.split(".")[0] < today:
                try:
                    # get sizes of the file and its archive
                    fileSize = int(subprocess.check_output("ls -l "+self.logDir+logFile+"|cut -f5 -d' '", shell=True))
                    archiveSize = int(subprocess.check_output("ssh "+self.archiveServer+" ls -l "+archiveDir+self.appName+"/"+logFile+"|cut -f5 -d' '", shell=True))
                    # only delete if the sizes are the same
                    if archiveSize == fileSize:
                        debug("debugPurgeData", "deleting", logFile)
                        os.remove(self.logDir+logFile)
                    else:
                        log("not deleting", self.logDir+logFile, "fileSize:", fileSize, "archiveSize:", archiveSize)
                except Exception as ex:
                    log("exception purging log file", logFile, str(ex))

# separate some parts of a camel case name by dots for easier parsing by the metrics server
def formatName(name, prefixes=[], suffixes=[]):
    newPrefix = ""
    for prefix in prefixes:
        if name[0:len(prefix)].lower() == prefix:
            newPrefix = prefix+"."
            name = name[len(prefix):]
            break
    newSuffix = ""
    for suffix in suffixes:
        if name[-len(suffix):].lower() == suffix:
            newSuffix = "."+suffix
            name = name[:-len(suffix)]
            break
    return newPrefix+name[0].lower()+name[1:]+newSuffix
# .replace(" ", "_")




from homealone import *
from socketserver import ThreadingMixIn
from http.server import HTTPServer
from http.server import BaseHTTPRequestHandler
import json
import urllib.parse
import threading
import socket
import time
import struct

# RESTful web services server interface
class RestServer(object):
    def __init__(self, name, resources=None, port=None, advert=True, event=None, label=""):
        debug('debugRestServer', name, "creating RestServer")
        self.name = name
        self.resources = resources
        self.advert = advert
        if event:
            self.event = event
        else:
            self.event = self.resources.event
        self.port = None
        if port:
            self.port = port
        else:
            # look for an available port in the pool
            restSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            for port in restServicePorts:
                try:
                    restSocket.bind(("", port))
                    self.port = port
                    debug('debugRestServer', "using port", port)
                    break
                except OSError:
                    pass
            restSocket.close()
        if self.port:
            if label == "":
                self.label = hostname+":"+str(self.port)
            else:
                self.label = label
            debug('debugInterrupt', self.label, "event", self.event)
            self.server = RestHTTPServer(('', self.port), RestRequestHandler, self)
            self.restAddr = multicastAddr
            self.stateSocket = None
            self.stateSequence = 0
        else:
            log("RestServer", "unable to find an available port")
            self.server = None

    def start(self):
        if not self.server:
            return
        debug('debugRestServer', self.name, "starting RestServer")
        # wait for the network to be available
        waitForDns()
        # start polling the resource states
        self.resources.start()
        if self.advert:
            # start the thread to send the resource states periodically and also when one changes
            def stateAdvert():
                debug('debugRestServer', self.name, "REST state started")
                resources = self.resources.dump()   # don't send expanded resources
                states = self.resources.getStates()
                lastStates = states
                self.stateTimeStamp = int(time.time())
                self.resourceTimeStamp = int(time.time())
                while True:
                    self.sendStateMessage(resources, states)
                    resources = None
                    states = None
                    # wait for either a state to change or the periodic trigger
                    currentStates = self.resources.getStates(wait=True)
                    # compare the current states to the previous states
                    if diffStates(lastStates, currentStates) != {}:
                        # a state changed
                        states = currentStates
                        self.stateTimeStamp = int(time.time())
                    if sorted(list(currentStates.keys())) != sorted(list(lastStates.keys())):
                        # a resource was either added or removed
                        resources = self.resources.dump()   # don't send expanded resources
                        self.resourceTimeStamp = int(time.time())
                    lastStates = currentStates
                debug('debugRestServer', self.name, "REST state ended")
            stateAdvertThread = LogThread(name="stateAdvertThread", target=stateAdvert)
            stateAdvertThread.start()

            # start the thread to trigger the advertisement message periodically
            def stateTrigger():
                debug('debugRestServer', self.name, "REST state trigger started", restAdvertInterval)
                while True:
                    debug('debugInterrupt', self.name, "trigger", "set", self.event)
                    self.event.set()
                    time.sleep(restAdvertInterval)
                debug('debugRestServer', self.name, "REST state trigger ended")
            stateTriggerThread = LogThread(name="stateTriggerThread", target=stateTrigger)
            stateTriggerThread.start()

        # start the HTTP server
        self.server.serve_forever()

    def openSocket(self):
        msgSocket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        msgSocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        return msgSocket

    def getServiceData(self):
        return {"name": self.name,
               "hostname": hostname,
               "port": self.port,
               "label": self.label,
               "statetimestamp": self.stateTimeStamp,
               "resourcetimestamp": self.resourceTimeStamp,
               "seq": self.stateSequence}

    def sendStateMessage(self, resources=None, states=None):
        stateMsg = {"service": self.getServiceData()}
        if resources:
            stateMsg["resources"] = resources
        if states:
            stateMsg["states"] = states
        if not self.stateSocket:
            self.stateSocket = self.openSocket()
        try:
            debug('debugRestState', self.name, str(list(stateMsg.keys())))
            self.stateSocket.sendto(bytes(json.dumps(stateMsg), "utf-8"),
                                                (self.restAddr, restAdvertPort))
        except socket.error as exception:
            log("socket error", str(exception))
            self.stateSocket = None
        self.stateSequence += 1


class RestHTTPServer(ThreadingMixIn, HTTPServer):
    def __init__(self, server_address, RequestHandlerClass, service):
        HTTPServer.__init__(self, server_address, RequestHandlerClass)
        self.service = service

class RestRequestHandler(BaseHTTPRequestHandler):
    serverVersion = "HARestHTTP/1.0"

    # return the resource definition or attribute specified in the path
    def do_GET(self):
        debug('debugRestGet', "verb:", "GET")
        debug('debugRestGet', "path:", self.path)
        debug('debugRestGet', "headers:", self.headers.__str__())
        try:
            (type, resName, attr, query) = self.parsePath(urllib.parse.unquote(self.path))
            service = self.server.service
            resources = service.resources
            data = None
            if type == "":              # no path specified
                data = ["service", "resources", "states"]
            elif type == "resources":   # resource definitions
                if resName:
                    try:                # resource was specified
                        resource = resources.getRes(resName, False)
                        if attr:        # attribute was specified
                             data = {attr: resource.__getattribute__(attr)}
                        else:           # no attribute, send resource definition
                             data = resource.dump()
                    except (KeyError, AttributeError):           # resource or attr not found
                        self.send_error(404)
                else:  # no resource was specified
                    expand = False
                    if query:   # expand resource definitions?
                        if (query[0][0].lower() == "expand") and (query[0][1].lower() == "true"):
                            expand = True
                    data = resources.dump(expand)
            elif type == "states":   # resource states
                data = resources.getStates()
            elif type == "service":  # service data
                data = service.getServiceData()
            else:
                self.send_error(404)
            if data:
                self.send_response(200)     # success
                self.send_header("Content-type", "application/json")
                self.end_headers()
                self.wfile.write(bytes(json.dumps(data), "utf-8"))
        except Exception as ex:
            logException("restServer do_GET "+self.path, ex)
            self.send_error(500)        # server error
            self.wfile.write(bytes(str(ex), "utf-8"))

    # set the attribute of the resource specified in the path to the value specified in the data
    def do_PUT(self):
        debug('debugRestPut', "verb:", "PUT")
        debug('debugRestPut', "path:", self.path)
        debug('debugRestPut', "headers:", self.headers.__str__())
        try:
            (type, resName, attr, query) = self.parsePath(urllib.parse.unquote(self.path))
            service = self.server.service
            resources = service.resources
            if (type == "resources") and resName and attr:   # resource and attr was specified
                try:
                    resource = resources.getRes(resName, False)
                    data = self.rfile.read(int(self.headers['Content-Length'])).decode("utf-8")
                    debug('debugRestPut', "data:", data)
                    if self.headers['Content-type'] == "application/json":
                        data = json.loads(data)
                    resource.__setattr__(attr, data[attr])
                    self.send_response(200) # success
                    self.end_headers()
                except (KeyError, AttributeError):           # resource or attr not found
                    raise
                    self.send_error(404)
            else:
                self.send_error(404)
        except Exception as ex:
            logException("restServer do_PUT "+self.path, ex)
            self.send_error(500)        # server error
            self.wfile.write(bytes(str(ex), "utf-8"))

    # add a resource to the collection specified in the path using parameters specified in the data
    def do_POST(self):
        self.send_error(501)         # not implemented

    # delete the resource specified in the path from the collection
    def do_DELETE(self):
        self.send_error(501)         # not implemented

    # parse the path string into components
    def parsePath(self, pathStr):
        try:
            (pathStr, queryStr) = pathStr.split("?")
            query = [queryItem.split("=") for queryItem in queryStr.split("&")]
        except ValueError:
            query = None
        segments = pathStr.lstrip("/").rstrip("/").split("/")
        type = segments[0]
        resource = None
        attr = None
        if len(segments) > 1:
            resource = segments[1]
            if len(segments) > 2:
                attr = segments[2]
        debug('debugRestParse', "elements:", type, resource, attr, query)
        return (type, resource, attr, query)

    # this suppresses logging from BaseHTTPServer
    def log_message(self, format, *args):
        return




import requests
import urllib.parse
from homealone import *

def sendNotification(notificationType, message):
    servers = findService("notification")
    if servers == []:
        log("notificationClient", "notification server not found")
    else:
        (host, port) = servers[0]
        url = "http://"+host+":"+port+"/notify?eventType="+notificationType+"&message="+urllib.parse.quote(message)
        request = requests.post(url)
        if request.status_code != 200:
            log("notificationClient", "error", url, request.status_code)




# sampleRate = .02

import time
import smbus
import threading
from homealone import *

# ADS1x15 analog digital converter

class ADS1x15Interface(Interface):
    def __init__(self, name, interface=None, addr=0x48, gain=4096, sps=250, icType=0x00, event=None):
        Interface.__init__(self, name, interface)
        # self.addr = addr
        self.gain = gain
        self.sps = sps
        self.icType = icType
        self.adc = ADS1x15(i2c=self.interface, addr=addr, icType=self.icType)
        self.lock = threading.Lock()
        self.event = event

    def read(self, addr):
        debug('debugAds', self.name, "read", addr, "gain", self.gain)
        try:
            with self.lock:
                # time.sleep(sampleRate)
                value = self.adc.readADCSingleEnded(addr, self.gain, self.sps)
                debug('debugAds', self.name, "value", value)
            return float(value / 1000)
        except Exception as ex:
            log("ADS1x15Interface exception", type(ex).__name__, str(ex))
            return 0

class ADS1x15DiffInterface(ADS1x15Interface):
    def __init__(self, name, interface=None, addr=0x48, gain=4096, sps=250, icType=0x00, event=None):
        ADS1x15Interface.__init__(self, name, interface, addr, gain, sps, icType)

    def read(self, addr):
        debug('debugAds', self.name, "read", addr, "gain", self.gain)
        try:
            with self.lock:
                # time.sleep(sampleRate)
                value = self.adc.readADCDifferential(addr, addr+1, self.gain, self.sps)
                debug('debugAds', self.name, "value", value)
            return float(value / 1000)
        except Exception as ex:
            log("ADS1x15DiffInterface exception", type(ex).__name__, str(ex))
            raise
            return 0

# ===========================================================================
# ADS1x15 Class
#
# Originally written by K. Townsend, Adafruit (https://github.com/adafruit/Adafruit-Raspberry-Pi-Python-Code/tree/master/Adafruit_ADS1x15)
# Updates and new functions implementation by Pedro Villanueva, 03/2013.
# The only error in the original code was in line 57:
#              __ADS1015_REG_CONFIG_DR_920SPS    = 0x0050
# should be
#              __ADS1015_REG_CONFIG_DR_920SPS    = 0x0060
#
# NOT IMPLEMENTED: Conversion ready pin, page 15 datasheet.
# ===========================================================================

class ADS1x15:
  i2c = None

  # icType Identifiers
  __IC_ADS1015                      = 0x00
  __IC_ADS1115                      = 0x01

  # Pointer Register
  __ADS1015_REG_POINTER_MASK        = 0x03
  __ADS1015_REG_POINTER_CONVERT     = 0x00
  __ADS1015_REG_POINTER_CONFIG      = 0x01
  __ADS1015_REG_POINTER_LOWTHRESH   = 0x02
  __ADS1015_REG_POINTER_HITHRESH    = 0x03

  # Config Register
  __ADS1015_REG_CONFIG_OS_MASK      = 0x8000
  __ADS1015_REG_CONFIG_OS_SINGLE    = 0x8000  # Write: Set to start a single-conversion
  __ADS1015_REG_CONFIG_OS_BUSY      = 0x0000  # Read: Bit = 0 when conversion is in progress
  __ADS1015_REG_CONFIG_OS_NOTBUSY   = 0x8000  # Read: Bit = 1 when device is not performing a conversion

  __ADS1015_REG_CONFIG_MUX_MASK     = 0x7000
  __ADS1015_REG_CONFIG_MUX_DIFF_0_1 = 0x0000  # Differential P = AIN0, N = AIN1 (default)
  __ADS1015_REG_CONFIG_MUX_DIFF_0_3 = 0x1000  # Differential P = AIN0, N = AIN3
  __ADS1015_REG_CONFIG_MUX_DIFF_1_3 = 0x2000  # Differential P = AIN1, N = AIN3
  __ADS1015_REG_CONFIG_MUX_DIFF_2_3 = 0x3000  # Differential P = AIN2, N = AIN3
  __ADS1015_REG_CONFIG_MUX_SINGLE_0 = 0x4000  # Single-ended AIN0
  __ADS1015_REG_CONFIG_MUX_SINGLE_1 = 0x5000  # Single-ended AIN1
  __ADS1015_REG_CONFIG_MUX_SINGLE_2 = 0x6000  # Single-ended AIN2
  __ADS1015_REG_CONFIG_MUX_SINGLE_3 = 0x7000  # Single-ended AIN3

  __ADS1015_REG_CONFIG_PGA_MASK     = 0x0E00
  __ADS1015_REG_CONFIG_PGA_6_144V   = 0x0000  # +/-6.144V range
  __ADS1015_REG_CONFIG_PGA_4_096V   = 0x0200  # +/-4.096V range
  __ADS1015_REG_CONFIG_PGA_2_048V   = 0x0400  # +/-2.048V range (default)
  __ADS1015_REG_CONFIG_PGA_1_024V   = 0x0600  # +/-1.024V range
  __ADS1015_REG_CONFIG_PGA_0_512V   = 0x0800  # +/-0.512V range
  __ADS1015_REG_CONFIG_PGA_0_256V   = 0x0A00  # +/-0.256V range

  __ADS1015_REG_CONFIG_MODE_MASK    = 0x0100
  __ADS1015_REG_CONFIG_MODE_CONTIN  = 0x0000  # Continuous conversion mode
  __ADS1015_REG_CONFIG_MODE_SINGLE  = 0x0100  # Power-down single-shot mode (default)

  __ADS1015_REG_CONFIG_DR_MASK      = 0x00E0
  __ADS1015_REG_CONFIG_DR_128SPS    = 0x0000  # 128 samples per second
  __ADS1015_REG_CONFIG_DR_250SPS    = 0x0020  # 250 samples per second
  __ADS1015_REG_CONFIG_DR_490SPS    = 0x0040  # 490 samples per second
  __ADS1015_REG_CONFIG_DR_920SPS    = 0x0060  # 920 samples per second
  __ADS1015_REG_CONFIG_DR_1600SPS   = 0x0080  # 1600 samples per second (default)
  __ADS1015_REG_CONFIG_DR_2400SPS   = 0x00A0  # 2400 samples per second
  __ADS1015_REG_CONFIG_DR_3300SPS   = 0x00C0  # 3300 samples per second (also 0x00E0)

  __ADS1115_REG_CONFIG_DR_8SPS      = 0x0000  # 8 samples per second
  __ADS1115_REG_CONFIG_DR_16SPS     = 0x0020  # 16 samples per second
  __ADS1115_REG_CONFIG_DR_32SPS     = 0x0040  # 32 samples per second
  __ADS1115_REG_CONFIG_DR_64SPS     = 0x0060  # 64 samples per second
  __ADS1115_REG_CONFIG_DR_128SPS    = 0x0080  # 128 samples per second
  __ADS1115_REG_CONFIG_DR_250SPS    = 0x00A0  # 250 samples per second (default)
  __ADS1115_REG_CONFIG_DR_475SPS    = 0x00C0  # 475 samples per second
  __ADS1115_REG_CONFIG_DR_860SPS    = 0x00E0  # 860 samples per second

  __ADS1015_REG_CONFIG_CMODE_MASK   = 0x0010
  __ADS1015_REG_CONFIG_CMODE_TRAD   = 0x0000  # Traditional comparator with hysteresis (default)
  __ADS1015_REG_CONFIG_CMODE_WINDOW = 0x0010  # Window comparator

  __ADS1015_REG_CONFIG_CPOL_MASK    = 0x0008
  __ADS1015_REG_CONFIG_CPOL_ACTVLOW = 0x0000  # ALERT/RDY pin is low when active (default)
  __ADS1015_REG_CONFIG_CPOL_ACTVHI  = 0x0008  # ALERT/RDY pin is high when active

  __ADS1015_REG_CONFIG_CLAT_MASK    = 0x0004  # Determines if ALERT/RDY pin latches once asserted
  __ADS1015_REG_CONFIG_CLAT_NONLAT  = 0x0000  # Non-latching comparator (default)
  __ADS1015_REG_CONFIG_CLAT_LATCH   = 0x0004  # Latching comparator

  __ADS1015_REG_CONFIG_CQUE_MASK    = 0x0003
  __ADS1015_REG_CONFIG_CQUE_1CONV   = 0x0000  # Assert ALERT/RDY after one conversions
  __ADS1015_REG_CONFIG_CQUE_2CONV   = 0x0001  # Assert ALERT/RDY after two conversions
  __ADS1015_REG_CONFIG_CQUE_4CONV   = 0x0002  # Assert ALERT/RDY after four conversions
  __ADS1015_REG_CONFIG_CQUE_NONE    = 0x0003  # Disable the comparator and put ALERT/RDY in high state (default)


  # Dictionaries with the sampling speed values
  # These simplify and clean the code (avoid the abuse of if/elif/else clauses)
  spsADS1115 = {
    8:__ADS1115_REG_CONFIG_DR_8SPS,
    16:__ADS1115_REG_CONFIG_DR_16SPS,
    32:__ADS1115_REG_CONFIG_DR_32SPS,
    64:__ADS1115_REG_CONFIG_DR_64SPS,
    128:__ADS1115_REG_CONFIG_DR_128SPS,
    250:__ADS1115_REG_CONFIG_DR_250SPS,
    475:__ADS1115_REG_CONFIG_DR_475SPS,
    860:__ADS1115_REG_CONFIG_DR_860SPS
  }
  spsADS1015 = {
    128:__ADS1015_REG_CONFIG_DR_128SPS,
    250:__ADS1015_REG_CONFIG_DR_250SPS,
    490:__ADS1015_REG_CONFIG_DR_490SPS,
    920:__ADS1015_REG_CONFIG_DR_920SPS,
    1600:__ADS1015_REG_CONFIG_DR_1600SPS,
    2400:__ADS1015_REG_CONFIG_DR_2400SPS,
    3300:__ADS1015_REG_CONFIG_DR_3300SPS
  }
  # Dictionariy with the programable gains
  pgaADS1x15 = {
    6144:__ADS1015_REG_CONFIG_PGA_6_144V,
    4096:__ADS1015_REG_CONFIG_PGA_4_096V,
    2048:__ADS1015_REG_CONFIG_PGA_2_048V,
    1024:__ADS1015_REG_CONFIG_PGA_1_024V,
    512:__ADS1015_REG_CONFIG_PGA_0_512V,
    256:__ADS1015_REG_CONFIG_PGA_0_256V
  }

  # Constructor
  def __init__(self, i2c, addr=0x48, icType=__IC_ADS1015, debug=False):
    # Depending on if you have an old or a new Raspberry Pi, you
    # may need to change the I2C bus.  Older Pis use SMBus 0,
    # whereas new Pis use SMBus 1.  If you see an error like:
    # 'Error accessing 0x48: Check your I2C address '
    # change the SMBus number in the initializer below!
    self.i2c = i2c
    self.addr = addr
    self.icType = icType
    self.debug = debug

    # Make sure the icType specified is valid
    if ((icType < self.__IC_ADS1015) | (icType > self.__IC_ADS1115)):
      if (self.debug):
        print("ADS1x15: Invalid icType specfied: %h" % icType)
      return -1
    else:
      self.icType = icType

    # Set pga value, so that getLastConversionResult() can use it,
    # any function that accepts a pga value must update this.
    self.pga = 6144


  def readADCSingleEnded(self, channel=0, pga=6144, sps=250):
    "Gets a single-ended ADC reading from the specified channel in mV. \
    The sample rate for this mode (single-shot) can be used to lower the noise \
    (low sps) or to lower the power consumption (high sps) by duty cycling, \
    see datasheet page 14 for more info. \
    The pga must be given in mV, see page 13 for the supported values."

    # With invalid channel return -1
    if (channel > 3):
      if (self.debug):
        print("ADS1x15: Invalid channel specified: %d" % channel)
      return -1

    # Disable comparator, Non-latching, Alert/Rdy active low
    # traditional comparator, single-shot mode
    config = self.__ADS1015_REG_CONFIG_CQUE_NONE    | \
             self.__ADS1015_REG_CONFIG_CLAT_NONLAT  | \
             self.__ADS1015_REG_CONFIG_CPOL_ACTVLOW | \
             self.__ADS1015_REG_CONFIG_CMODE_TRAD   | \
             self.__ADS1015_REG_CONFIG_MODE_SINGLE

    # Set sample per seconds, defaults to 250sps
    # If sps is in the dictionary (defined in init) it returns the value of the constant
    # othewise it returns the value for 250sps. This saves a lot of if/elif/else code!
    if (self.icType == self.__IC_ADS1015):
      config |= self.spsADS1015.setdefault(sps, self.__ADS1015_REG_CONFIG_DR_1600SPS)
    else:
      if ( (sps not in self.spsADS1115) & self.debug):
        print("ADS1x15: Invalid pga specified: %d, using 6144mV" % sps)
      config |= self.spsADS1115.setdefault(sps, self.__ADS1115_REG_CONFIG_DR_250SPS)

    # Set PGA/voltage range, defaults to +-6.144V
    if ( (pga not in self.pgaADS1x15) & self.debug):
      print("ADS1x15: Invalid pga specified: %d, using 6144mV" % sps)
    config |= self.pgaADS1x15.setdefault(pga, self.__ADS1015_REG_CONFIG_PGA_6_144V)
    self.pga = pga

    # Set the channel to be converted
    if channel == 3:
      config |= self.__ADS1015_REG_CONFIG_MUX_SINGLE_3
    elif channel == 2:
      config |= self.__ADS1015_REG_CONFIG_MUX_SINGLE_2
    elif channel == 1:
      config |= self.__ADS1015_REG_CONFIG_MUX_SINGLE_1
    else:
      config |= self.__ADS1015_REG_CONFIG_MUX_SINGLE_0

    # Set 'start single-conversion' bit
    config |= self.__ADS1015_REG_CONFIG_OS_SINGLE

    # Write config register to the ADC
    bytes = [(config >> 8) & 0xFF, config & 0xFF]
    self.i2c.writeBlock(self.addr, self.__ADS1015_REG_POINTER_CONFIG, bytes)

    # Wait for the ADC conversion to complete
    # The minimum delay depends on the sps: delay >= 1/sps
    # We add 0.1ms to be sure
    delay = 1.0/sps+0.0001
    time.sleep(delay)

    # Read the conversion results
    result = self.i2c.readBlock(self.addr, self.__ADS1015_REG_POINTER_CONVERT, 2)
    if (self.icType == self.__IC_ADS1015):
        # Shift right 4 bits for the 12-bit ADS1015 and convert to mV
        return ( ((result[0] << 8) | (result[1] & 0xFF)) >> 4 )*pga/2048.0
    else:
      # Return a mV value for the ADS1115
      # (Take signed values into account as well)
      val = (result[0] << 8) | (result[1])
      if val > 0x7FFF:
        return (val - 0xFFFF)*pga/32768.0
      else:
        return ( (result[0] << 8) | (result[1]) )*pga/32768.0


  def readADCDifferential(self, chP=0, chN=1, pga=6144, sps=250):
    "Gets a differential ADC reading from channels chP and chN in mV. \
    The sample rate for this mode (single-shot) can be used to lower the noise \
    (low sps) or to lower the power consumption (high sps) by duty cycling, \
    see data sheet page 14 for more info. \
    The pga must be given in mV, see page 13 for the supported values."

    # Disable comparator, Non-latching, Alert/Rdy active low
    # traditional comparator, single-shot mode
    config = self.__ADS1015_REG_CONFIG_CQUE_NONE    | \
             self.__ADS1015_REG_CONFIG_CLAT_NONLAT  | \
             self.__ADS1015_REG_CONFIG_CPOL_ACTVLOW | \
             self.__ADS1015_REG_CONFIG_CMODE_TRAD   | \
             self.__ADS1015_REG_CONFIG_MODE_SINGLE

    # Set channels
    if ( (chP == 0) & (chN == 1) ):
      config |= self.__ADS1015_REG_CONFIG_MUX_DIFF_0_1
    elif ( (chP == 0) & (chN == 3) ):
      config |= self.__ADS1015_REG_CONFIG_MUX_DIFF_0_3
    elif ( (chP == 2) & (chN == 3) ):
      config |= self.__ADS1015_REG_CONFIG_MUX_DIFF_2_3
    elif ( (chP == 1) & (chN == 3) ):
      config |= self.__ADS1015_REG_CONFIG_MUX_DIFF_1_3
    else:
      if (self.debug):
        print("ADS1x15: Invalid channels specified: %d, %d" % (chP, chN))
        return -1

    # Set sample per seconds, defaults to 250sps
    # If sps is in the dictionary (defined in init()) it returns the value of the constant
    # othewise it returns the value for 250sps. This saves a lot of if/elif/else code!
    if (self.icType == self.__IC_ADS1015):
      config |= self.spsADS1015.setdefault(sps, self.__ADS1015_REG_CONFIG_DR_1600SPS)
    else:
      if ( (sps not in self.spsADS1115) & self.debug):
        print("ADS1x15: Invalid pga specified: %d, using 6144mV" % sps)
      config |= self.spsADS1115.setdefault(sps, self.__ADS1115_REG_CONFIG_DR_250SPS)

    # Set PGA/voltage range, defaults to +-6.144V
    if ( (pga not in self.pgaADS1x15) & self.debug):
      print("ADS1x15: Invalid pga specified: %d, using 6144mV" % sps)
    config |= self.pgaADS1x15.setdefault(pga, self.__ADS1015_REG_CONFIG_PGA_6_144V)
    self.pga = pga

    # Set 'start single-conversion' bit
    config |= self.__ADS1015_REG_CONFIG_OS_SINGLE

    # Write config register to the ADC
    bytes = [(config >> 8) & 0xFF, config & 0xFF]
    self.i2c.writeBlock(self.addr, self.__ADS1015_REG_POINTER_CONFIG, bytes)

    # Wait for the ADC conversion to complete
    # The minimum delay depends on the sps: delay >= 1/sps
    # We add 0.1ms to be sure
    delay = 1.0/sps+0.0001
    time.sleep(delay)

    # Read the conversion results
    result = self.i2c.readBlock(self.addr, self.__ADS1015_REG_POINTER_CONVERT, 2)
    if (self.icType == self.__IC_ADS1015):
        # Shift right 4 bits for the 12-bit ADS1015 and convert to mV
        return ( ((result[0] << 8) | (result[1] & 0xFF)) >> 4 )*pga/2048.0
    else:
      # Return a mV value for the ADS1115
      # (Take signed values into account as well)
      val = (result[0] << 8) | (result[1])
      if val > 0x7FFF:
        return (val - 0xFFFF)*pga/32768.0
      else:
        return ( (result[0] << 8) | (result[1]) )*pga/32768.0


  def readADCDifferential01(self, pga=6144, sps=250):
    "Gets a differential ADC reading from channels 0 and 1 in mV\
    The sample rate for this mode (single-shot) can be used to lower the noise \
    (low sps) or to lower the power consumption (high sps) by duty cycling, \
    see data sheet page 14 for more info. \
    The pga must be given in mV, see page 13 for the supported values."
    return self.readADCDifferential(0, 1, pga, sps)


  def readADCDifferential03(self, pga=6144, sps=250):
    "Gets a differential ADC reading from channels 0 and 3 in mV \
    The sample rate for this mode (single-shot) can be used to lower the noise \
    (low sps) or to lower the power consumption (high sps) by duty cycling, \
    see data sheet page 14 for more info. \
    The pga must be given in mV, see page 13 for the supported values."
    return self.readADCDifferential(0, 3, pga, sps)


  def readADCDifferential13(self, pga=6144, sps=250):
    "Gets a differential ADC reading from channels 1 and 3 in mV \
    The sample rate for this mode (single-shot) can be used to lower the noise \
    (low sps) or to lower the power consumption (high sps) by duty cycling, \
    see data sheet page 14 for more info. \
    The pga must be given in mV, see page 13 for the supported values."
    return self.__readADCDifferential(1, 3, pga, sps)


  def readADCDifferential23(self, pga=6144, sps=250):
    "Gets a differential ADC reading from channels 2 and 3 in mV \
    The sample rate for this mode (single-shot) can be used to lower the noise \
    (low sps) or to lower the power consumption (high sps) by duty cycling, \
    see data sheet page 14 for more info. \
    The pga must be given in mV, see page 13 for the supported values."
    return self.readADCDifferential(2, 3, pga, sps)


  def startContinuousConversion(self, channel=0, pga=6144, sps=250):
    "Starts the continuous conversion mode and returns the first ADC reading \
    in mV from the specified channel. \
    The sps controls the sample rate. \
    The pga must be given in mV, see datasheet page 13 for the supported values. \
    Use getLastConversionResults() to read the next values and \
    stopContinuousConversion() to stop converting."

    # Default to channel 0 with invalid channel, or return -1?
    if (channel > 3):
      if (self.debug):
        print("ADS1x15: Invalid channel specified: %d" % channel)
      return -1

    # Disable comparator, Non-latching, Alert/Rdy active low
    # traditional comparator, continuous mode
    # The last flag is the only change we need, page 11 datasheet
    config = self.__ADS1015_REG_CONFIG_CQUE_NONE    | \
             self.__ADS1015_REG_CONFIG_CLAT_NONLAT  | \
             self.__ADS1015_REG_CONFIG_CPOL_ACTVLOW | \
             self.__ADS1015_REG_CONFIG_CMODE_TRAD   | \
             self.__ADS1015_REG_CONFIG_MODE_CONTIN

    # Set sample per seconds, defaults to 250sps
    # If sps is in the dictionary (defined in init()) it returns the value of the constant
    # othewise it returns the value for 250sps. This saves a lot of if/elif/else code!
    if (self.icType == self.__IC_ADS1015):
      config |= self.spsADS1015.setdefault(sps, self.__ADS1015_REG_CONFIG_DR_1600SPS)
    else:
      if ( (sps not in self.spsADS1115) & self.debug):
        print("ADS1x15: Invalid pga specified: %d, using 6144mV" % sps)
      config |= self.spsADS1115.setdefault(sps, self.__ADS1115_REG_CONFIG_DR_250SPS)

    # Set PGA/voltage range, defaults to +-6.144V
    if ( (pga not in self.pgaADS1x15) & self.debug):
      print("ADS1x15: Invalid pga specified: %d, using 6144mV" % sps)
    config |= self.pgaADS1x15.setdefault(pga, self.__ADS1015_REG_CONFIG_PGA_6_144V)
    self.pga = pga

    # Set the channel to be converted
    if channel == 3:
      config |= self.__ADS1015_REG_CONFIG_MUX_SINGLE_3
    elif channel == 2:
      config |= self.__ADS1015_REG_CONFIG_MUX_SINGLE_2
    elif channel == 1:
      config |= self.__ADS1015_REG_CONFIG_MUX_SINGLE_1
    else:
      config |= self.__ADS1015_REG_CONFIG_MUX_SINGLE_0

    # Set 'start single-conversion' bit to begin conversions
    # No need to change this for continuous mode!
    config |= self.__ADS1015_REG_CONFIG_OS_SINGLE

    # Write config register to the ADC
    # Once we write the ADC will convert continously
    # we can read the next values using getLastConversionResult
    bytes = [(config >> 8) & 0xFF, config & 0xFF]
    self.i2c.writeBlock(self.addr, self.__ADS1015_REG_POINTER_CONFIG, bytes)

    # Wait for the ADC conversion to complete
    # The minimum delay depends on the sps: delay >= 1/sps
    # We add 0.5ms to be sure
    delay = 1.0/sps+0.0005
    time.sleep(delay)

    # Read the conversion results
    result = self.i2c.readBlock(self.addr, self.__ADS1015_REG_POINTER_CONVERT, 2)
    if (self.icType == self.__IC_ADS1015):
        # Shift right 4 bits for the 12-bit ADS1015 and convert to mV
        return ( ((result[0] << 8) | (result[1] & 0xFF)) >> 4 )*pga/2048.0
    else:
      # Return a mV value for the ADS1115
      # (Take signed values into account as well)
      val = (result[0] << 8) | (result[1])
      if val > 0x7FFF:
        return (val - 0xFFFF)*pga/32768.0
      else:
        return ( (result[0] << 8) | (result[1]) )*pga/32768.0

  def startContinuousDifferentialConversion(self, chP=0, chN=1, pga=6144, sps=250):
    "Starts the continuous differential conversion mode and returns the first ADC reading \
    in mV as the difference from the specified channels. \
    The sps controls the sample rate. \
    The pga must be given in mV, see datasheet page 13 for the supported values. \
    Use getLastConversionResults() to read the next values and \
    stopContinuousConversion() to stop converting."

    # Disable comparator, Non-latching, Alert/Rdy active low
    # traditional comparator, continuous mode
    # The last flag is the only change we need, page 11 datasheet
    config = self.__ADS1015_REG_CONFIG_CQUE_NONE    | \
             self.__ADS1015_REG_CONFIG_CLAT_NONLAT  | \
             self.__ADS1015_REG_CONFIG_CPOL_ACTVLOW | \
             self.__ADS1015_REG_CONFIG_CMODE_TRAD   | \
             self.__ADS1015_REG_CONFIG_MODE_CONTIN

    # Set sample per seconds, defaults to 250sps
    # If sps is in the dictionary (defined in init()) it returns the value of the constant
    # othewise it returns the value for 250sps. This saves a lot of if/elif/else code!
    if (self.icType == self.__IC_ADS1015):
      config |= self.spsADS1015.setdefault(sps, self.__ADS1015_REG_CONFIG_DR_1600SPS)
    else:
      if ( (sps not in self.spsADS1115) & self.debug):
        print("ADS1x15: Invalid pga specified: %d, using 6144mV" % sps)
      config |= self.spsADS1115.setdefault(sps, self.__ADS1115_REG_CONFIG_DR_250SPS)

    # Set PGA/voltage range, defaults to +-6.144V
    if ( (pga not in self.pgaADS1x15) & self.debug):
      print("ADS1x15: Invalid pga specified: %d, using 6144mV" % sps)
    config |= self.pgaADS1x15.setdefault(pga, self.__ADS1015_REG_CONFIG_PGA_6_144V)
    self.pga = pga

    # Set channels
    if ( (chP == 0) & (chN == 1) ):
      config |= self.__ADS1015_REG_CONFIG_MUX_DIFF_0_1
    elif ( (chP == 0) & (chN == 3) ):
      config |= self.__ADS1015_REG_CONFIG_MUX_DIFF_0_3
    elif ( (chP == 2) & (chN == 3) ):
      config |= self.__ADS1015_REG_CONFIG_MUX_DIFF_2_3
    elif ( (chP == 1) & (chN == 3) ):
      config |= self.__ADS1015_REG_CONFIG_MUX_DIFF_1_3
    else:
      if (self.debug):
        print("ADS1x15: Invalid channels specified: %d, %d" % (chP, chN))
    return -1

    # Set 'start single-conversion' bit to begin conversions
    # No need to change this for continuous mode!
    config |= self.__ADS1015_REG_CONFIG_OS_SINGLE

    # Write config register to the ADC
    # Once we write the ADC will convert continously
    # we can read the next values using getLastConversionResult
    bytes = [(config >> 8) & 0xFF, config & 0xFF]
    self.i2c.writeBlock(self.addr, self.__ADS1015_REG_POINTER_CONFIG, bytes)

    # Wait for the ADC conversion to complete
    # The minimum delay depends on the sps: delay >= 1/sps
    # We add 0.5ms to be sure
    delay = 1.0/sps+0.0005
    time.sleep(delay)

    # Read the conversion results
    result = self.i2c.readBlock(self.addr, self.__ADS1015_REG_POINTER_CONVERT, 2)
    if (self.icType == self.__IC_ADS1015):
      # Shift right 4 bits for the 12-bit ADS1015 and convert to mV
      return ( ((result[0] << 8) | (result[1] & 0xFF)) >> 4 )*pga/2048.0
    else:
      # Return a mV value for the ADS1115
      # (Take signed values into account as well)
      val = (result[0] << 8) | (result[1])
      if val > 0x7FFF:
        return (val - 0xFFFF)*pga/32768.0
      else:
        return ( (result[0] << 8) | (result[1]) )*pga/32768.0


  def stopContinuousConversion(self):
    "Stops the ADC's conversions when in continuous mode \
    and resets the configuration to its default value."
    # Write the default config register to the ADC
    # Once we write, the ADC will do a single conversion and
    # enter power-off mode.
    config = 0x8583 # Page 18 datasheet.
    bytes = [(config >> 8) & 0xFF, config & 0xFF]
    self.i2c.writeBlock(self.addr, self.__ADS1015_REG_POINTER_CONFIG, bytes)
    return True

  def getLastConversionResults(self):
    "Returns the last ADC conversion result in mV"
    # Read the conversion results
    result = self.i2c.readBlock(self.addr, self.__ADS1015_REG_POINTER_CONVERT, 2)
    if (self.icType == self.__IC_ADS1015):
        # Shift right 4 bits for the 12-bit ADS1015 and convert to mV
        return ( ((result[0] << 8) | (result[1] & 0xFF)) >> 4 )*self.pga/2048.0
    else:
      # Return a mV value for the ADS1115
      # (Take signed values into account as well)
      val = (result[0] << 8) | (result[1])
      if val > 0x7FFF:
        return (val - 0xFFFF)*self.pga/32768.0
      else:
        return ( (result[0] << 8) | (result[1]) )*self.pga/32768.0


  def startSingleEndedComparator(self, channel, thresholdHigh, thresholdLow, \
                                 pga=6144, sps=250, \
                                 activeLow=True, traditionalMode=True, latching=False, \
                                 numReadings=1):
    "Starts the comparator mode on the specified channel, see datasheet pg. 15. \
    In traditional mode it alerts (ALERT pin will go low)  when voltage exceeds  \
    thresholdHigh until it falls below thresholdLow (both given in mV). \
    In window mode (traditionalMode=False) it alerts when voltage doesn't lie\
    between both thresholds.\
    In latching mode the alert will continue until the conversion value is read. \
    numReadings controls how many readings are necessary to trigger an alert: 1, 2 or 4.\
    Use getLastConversionResults() to read the current value  (which may differ \
    from the one that triggered the alert) and clear the alert pin in latching mode. \
    This function starts the continuous conversion mode.  The sps controls \
    the sample rate and the pga the gain, see datasheet page 13. "

    # With invalid channel return -1
    if (channel > 3):
      if (self.debug):
        print("ADS1x15: Invalid channel specified: %d" % channel)
      return -1

    # Continuous mode
    config = self.__ADS1015_REG_CONFIG_MODE_CONTIN

    if (activeLow==False):
      config |= self.__ADS1015_REG_CONFIG_CPOL_ACTVHI
    else:
      config |= self.__ADS1015_REG_CONFIG_CPOL_ACTVLOW

    if (traditionalMode==False):
      config |= self.__ADS1015_REG_CONFIG_CMODE_WINDOW
    else:
      config |= self.__ADS1015_REG_CONFIG_CMODE_TRAD

    if (latching==True):
      config |= self.__ADS1015_REG_CONFIG_CLAT_LATCH
    else:
      config |= self.__ADS1015_REG_CONFIG_CLAT_NONLAT

    if (numReadings==4):
      config |= self.__ADS1015_REG_CONFIG_CQUE_4CONV
    elif (numReadings==2):
      config |= self.__ADS1015_REG_CONFIG_CQUE_2CONV
    else:
      config |= self.__ADS1015_REG_CONFIG_CQUE_1CONV

    # Set sample per seconds, defaults to 250sps
    # If sps is in the dictionary (defined in init()) it returns the value of the constant
    # othewise it returns the value for 250sps. This saves a lot of if/elif/else code!
    if (self.icType == self.__IC_ADS1015):
      if ( (sps not in self.spsADS1015) & self.debug):
        print("ADS1x15: Invalid sps specified: %d, using 1600sps" % sps)
      config |= self.spsADS1015.setdefault(sps, self.__ADS1015_REG_CONFIG_DR_1600SPS)
    else:
      if ( (sps not in self.spsADS1115) & self.debug):
        print("ADS1x15: Invalid sps specified: %d, using 250sps" % sps)
      config |= self.spsADS1115.setdefault(sps, self.__ADS1115_REG_CONFIG_DR_250SPS)

    # Set PGA/voltage range, defaults to +-6.144V
    if ( (pga not in self.pgaADS1x15) & self.debug):
      print("ADS1x15: Invalid pga specified: %d, using 6144mV" % pga)
    config |= self.pgaADS1x15.setdefault(pga, self.__ADS1015_REG_CONFIG_PGA_6_144V)
    self.pga = pga

    # Set the channel to be converted
    if channel == 3:
      config |= self.__ADS1015_REG_CONFIG_MUX_SINGLE_3
    elif channel == 2:
      config |= self.__ADS1015_REG_CONFIG_MUX_SINGLE_2
    elif channel == 1:
      config |= self.__ADS1015_REG_CONFIG_MUX_SINGLE_1
    else:
      config |= self.__ADS1015_REG_CONFIG_MUX_SINGLE_0

    # Set 'start single-conversion' bit to begin conversions
    config |= self.__ADS1015_REG_CONFIG_OS_SINGLE

    # Write threshold high and low registers to the ADC
    # V_digital = (2^(n-1)-1)/pga*V_analog
    if (self.icType == self.__IC_ADS1015):
      thresholdHighWORD = int(thresholdHigh*(2048.0/pga))
    else:
      thresholdHighWORD = int(thresholdHigh*(32767.0/pga))
    bytes = [(thresholdHighWORD >> 8) & 0xFF, thresholdHighWORD & 0xFF]
    self.i2c.writeBlock(self.addr, self.__ADS1015_REG_POINTER_HITHRESH, bytes)

    if (self.icType == self.__IC_ADS1015):
      thresholdLowWORD = int(thresholdLow*(2048.0/pga))
    else:
      thresholdLowWORD = int(thresholdLow*(32767.0/pga))
    bytes = [(thresholdLowWORD >> 8) & 0xFF, thresholdLowWORD & 0xFF]
    self.i2c.writeBlock(self.addr, self.__ADS1015_REG_POINTER_LOWTHRESH, bytes)

    # Write config register to the ADC
    # Once we write the ADC will convert continously and alert when things happen,
    # we can read the converted values using getLastConversionResult
    bytes = [(config >> 8) & 0xFF, config & 0xFF]
    self.i2c.writeBlock(self.addr, self.__ADS1015_REG_POINTER_CONFIG, bytes)


  def startDifferentialComparator(self, chP, chN, thresholdHigh, thresholdLow, \
                                 pga=6144, sps=250, \
                                 activeLow=True, traditionalMode=True, latching=False, \
                                 numReadings=1):
    "Starts the comparator mode on the specified channel, see datasheet pg. 15. \
    In traditional mode it alerts (ALERT pin will go low)  when voltage exceeds  \
    thresholdHigh until it falls below thresholdLow (both given in mV). \
    In window mode (traditionalMode=False) it alerts when voltage doesn't lie\
    between both thresholds.\
    In latching mode the alert will continue until the conversion value is read. \
    numReadings controls how many readings are necessary to trigger an alert: 1, 2 or 4.\
    Use getLastConversionResults() to read the current value  (which may differ \
    from the one that triggered the alert) and clear the alert pin in latching mode. \
    This function starts the continuous conversion mode.  The sps controls \
    the sample rate and the pga the gain, see datasheet page 13. "

    # Continuous mode
    config = self.__ADS1015_REG_CONFIG_MODE_CONTIN

    if (activeLow==False):
      config |= self.__ADS1015_REG_CONFIG_CPOL_ACTVHI
    else:
      config |= self.__ADS1015_REG_CONFIG_CPOL_ACTVLOW

    if (traditionalMode==False):
      config |= self.__ADS1015_REG_CONFIG_CMODE_WINDOW
    else:
      config |= self.__ADS1015_REG_CONFIG_CMODE_TRAD

    if (latching==True):
      config |= self.__ADS1015_REG_CONFIG_CLAT_LATCH
    else:
      config |= self.__ADS1015_REG_CONFIG_CLAT_NONLAT

    if (numReadings==4):
      config |= self.__ADS1015_REG_CONFIG_CQUE_4CONV
    elif (numReadings==2):
      config |= self.__ADS1015_REG_CONFIG_CQUE_2CONV
    else:
      config |= self.__ADS1015_REG_CONFIG_CQUE_1CONV

    # Set sample per seconds, defaults to 250sps
    # If sps is in the dictionary (defined in init()) it returns the value of the constant
    # othewise it returns the value for 250sps. This saves a lot of if/elif/else code!
    if (self.icType == self.__IC_ADS1015):
      if ( (sps not in self.spsADS1015) & self.debug):
        print("ADS1x15: Invalid sps specified: %d, using 1600sps" % sps)
      config |= self.spsADS1015.setdefault(sps, self.__ADS1015_REG_CONFIG_DR_1600SPS)
    else:
      if ( (sps not in self.spsADS1115) & self.debug):
        print("ADS1x15: Invalid sps specified: %d, using 250sps" % sps)
      config |= self.spsADS1115.setdefault(sps, self.__ADS1115_REG_CONFIG_DR_250SPS)

    # Set PGA/voltage range, defaults to +-6.144V
    if ( (pga not in self.pgaADS1x15) & self.debug):
      print("ADS1x15: Invalid pga specified: %d, using 6144mV" % pga)
    config |= self.pgaADS1x15.setdefault(pga, self.__ADS1015_REG_CONFIG_PGA_6_144V)
    self.pga = pga

    # Set channels
    if ( (chP == 0) & (chN == 1) ):
      config |= self.__ADS1015_REG_CONFIG_MUX_DIFF_0_1
    elif ( (chP == 0) & (chN == 3) ):
      config |= self.__ADS1015_REG_CONFIG_MUX_DIFF_0_3
    elif ( (chP == 2) & (chN == 3) ):
      config |= self.__ADS1015_REG_CONFIG_MUX_DIFF_2_3
    elif ( (chP == 1) & (chN == 3) ):
      config |= self.__ADS1015_REG_CONFIG_MUX_DIFF_1_3
    else:
      if (self.debug):
        print("ADS1x15: Invalid channels specified: %d, %d" % (chP, chN))
    return -1

    # Set 'start single-conversion' bit to begin conversions
    config |= self.__ADS1015_REG_CONFIG_OS_SINGLE

    # Write threshold high and low registers to the ADC
    # V_digital = (2^(n-1)-1)/pga*V_analog
    if (self.icType == self.__IC_ADS1015):
      thresholdHighWORD = int(thresholdHigh*(2048.0/pga))
    else:
      thresholdHighWORD = int(thresholdHigh*(32767.0/pga))
    bytes = [(thresholdHighWORD >> 8) & 0xFF, thresholdHighWORD & 0xFF]
    self.i2c.writeBlock(self.addr, self.__ADS1015_REG_POINTER_HITHRESH, bytes)

    if (self.icType == self.__IC_ADS1015):
      thresholdLowWORD = int(thresholdLow*(2048.0/pga))
    else:
      thresholdLowWORD = int(thresholdLow*(32767.0/pga))
    bytes = [(thresholdLowWORD >> 8) & 0xFF, thresholdLowWORD & 0xFF]
    self.i2c.writeBlock(self.addr, self.__ADS1015_REG_POINTER_LOWTHRESH, bytes)

    # Write config register to the ADC
    # Once we write the ADC will convert continously and alert when things happen,
    # we can read the converted values using getLastConversionResult
    bytes = [(config >> 8) & 0xFF, config & 0xFF]
    self.i2c.writeBlock(self.addr, self.__ADS1015_REG_POINTER_CONFIG, bytes)



from homealone import *
import time
import os
import datetime
import pytz

class TimeInterface(Interface):
    def __init__(self, name, interface=None, event=None, clock=None, latLong=None, tz=None):
        Interface.__init__(self, name, interface, event)
        self.clock = None
        # interface: optional gps location data file
        if interface:
            self.clock = "gps"
            import timezonefinder
            self.timeZoneFinder = timezonefinder.TimezoneFinder()
        # clock: time source
        #   local - system clock is local tz (default)
        #   utc - system clock is UTC
        #   gps - gps time via file interface
        if clock:
            self.clock = clock
        else:
            if not self.clock:
                self.clock = "local"
        # latLong: location coordinates as tuple(lat, long)
        if latLong:
            self.latLong = latLong
        else:
            self.latLong = (0, 0)
        # tz: time zone in Olson format (e.g. "America/Los_Angeles")
        if tz:
            self.tz = tz
        else:
            if self.clock == "local":   # local TZ if clock is local
                self.tz = '/'.join(os.readlink('/etc/localtime').split('/')[-2:])
            else:
                self.tz = "UTC"

        debug('debugTime', self.name, "clock:", self.clock, "latLong:", self.latLong, "tz:", self.tz)

    def read(self, addr=None):
        # get gps location and figure out the time zone
        if self.interface:
            self.latLong = (self.interface.read("Lat"), self.interface.read("Long"))
            if self.latLong == (0, 0):
                self.tz = "UTC"
            else:
                self.tz = self.timeZoneFinder.timezone_at(lat=self.latLong[0], lng=self.latLong[1])
            debug('debugTime', self.name, "lat:", self.latLong[0], "long:", self.latLong[1], "tz:", self.tz)
        # get the naive UTC time
        if self.clock == "gps":                 # from file
            utcNow = datetime.datetime(*time.strptime(self.interface.read("Time"), "%Y-%m-%d %H:%M:%S")[0:6])
        elif self.clock in ["utc", "local"]:    # from system clock
            utcNow = datetime.datetime.utcnow()
        # convert to TZ aware local time
        localNow = utcNow.replace(tzinfo=pytz.utc).astimezone(pytz.timezone(self.tz))
        debug('debugTime', self.name, "now:", localNow, "tz:", self.tz, "latLong:", self.latLong)
        # return the requested item
        if addr:
            if addr == "daylight":
                return normalState(sunIsUp(localNow, self.latLong))
            elif addr == "sunrise":
                return sunrise(localNow, self.latLong).strftime("%I:%M %p").lstrip("0")
            elif addr == "sunset":
                return sunset(localNow, self.latLong).strftime("%I:%M %p").lstrip("0")
            elif addr == "timeZone":
                return self.tz
            elif addr == "timeZoneName":
                try:
                    return pytz.timezone(self.tz).tzname(utcNow)
                except pytz.exceptions.AmbiguousTimeError:
                    return "WTF"
            else:
                return time.strftime(addr, localNow.timetuple()).lstrip("0")
        else:
            return time.asctime()



import serial
import sys
from homealone import *

class SerialInterface(Interface):
    def __init__(self, name, interface=None, event=None, device="", config={}):
        Interface.__init__(self, name, interface=interface, event=event)
        self.device = device
        self.config = config

    def start(self):
        if self.device == "/dev/stdin":
            log(self.name, "using stdin", self.device)
            self.inPort = sys.stdin
            self.outPort = sys.stdout
        else:
            try:
                log(self.name, "opening serial port", self.device)
                self.inPort = self.outPort = serial.Serial(self.device, **self.config)
            except:
                log(self.name, "unable to open serial port")
                return

    def stop(self):
        if self.device != "/dev/stdin":
            self.inPort.close()

    def read(self, addr, theLen=1):
        return self.inPort.read(theLen)

    def write(self, addr, value):
        self.outPort.write(value)

    def readline(self, addr):
        return self.inPort.readline()

    def writeline(self, addr, value):
        self.outPort.writeline(value)



# initialize the global variables that define the environment
import os
import socket
import sys
import inspect

# network
hostname = socket.gethostname()
localController = "localhost"
externalUrl = "example.com"

# directory structure
# directory containing this application
appDir = sys.path[0]
# site configuration directory - add to path so config can be imported
siteDir = "/etc/homealone/site"
sys.path.insert(0, siteDir)
keyDir = siteDir+"/keys/"       # external credentials
stateDir = appDir+"/state/"     # app specific states
dataDir = appDir+"/data/"       # data logging

# logging and debugging
sysLogging = True
debugEnable = False

# Localization
latLong = (0.0, 0.0)
elevation = 0 # elevation in feet
tempScale = "F"
defaultCountryCode = "1"
defaultAreaCode = "000"

# Data logging and metrics
metricsPrefix = "com.example.ha"
metricsHost = "metrics.example.com"
metricsPort = 2003
sendMetrics = True
logData = True
archiveData = True
purgeData = True
purgeDays = 5
logChanged = True
archiveService = "archive"
archiveDir = "/archives/"

# REST parameters
restServicePorts = [7378, 7377, 7376, 7375, 7374, 7373, 7372, 7371]
restServicePort = 7378
restAdvertPort = 4244 # 7379
restNotificationPort = 7370
ipv4MulticastAddr = "224.0.0.1"
ipv6MulticastAddr = "ff02::1"
multicastAddr = ipv4MulticastAddr
restAdvertInterval = 10
restTimeout = 60
restRetryInterval = 10

# Alerts and events
alertConfig = {}
eventConfig = {}

# optionally import site configuration
try:
    from siteConf import *
except ImportError:
    pass

# optionally import app configuration
try:
    from conf import *
except ImportError:
    pass



from homealone import *
# import whichever gpio library is installed
try:
    import RPi.GPIO as gpio
    gpioLibrary = "RPi.GPIO"
except ImportError:
    import RPIO as gpio
    gpioLibrary = "RPIO"

# dictionary of MCP23017Interfaces indexed by their interrupt pins
gpioInterfaces = {}

# initial interrupt callback routine that is called when an interrupt pin goes low
def interruptCallback(pin, value=1):
    debug('debugGPIO', "interruptCallback", "pin:", pin, "value:", value)
    try:
        # call the interrupt routine for the MCP23017Interface associated with the pin
        gpioInterfaces[pin].interrupt()
    except KeyError:
        # the interrupt occurred on a pin not associated with a MCP23017Interface
        log("interruptCallback", "unknown interrupt", "pin:", pin, "value:", value, "gpioInterfaces:", gpioInterfaces)


# Interface to GPIO via MCP23017 I2C I/O expander
class MCP23017Interface(Interface):
    # MCP23017 I2C I/O expander
    IODIR = 0x00        # I/O direction
    IPOL = 0x02         # input polarity
    GPINTEN = 0x04      # interrupt on change
    DEFVAL = 0x06       # default value
    INTCON = 0x08       # interrupt control
    IOCON = 0x0a        # configuration
    GPPU = 0x0c         # pull up resistor
    INTF = 0x0e         # interrupt flag
    INTCAP = 0x10       # interrupt capture
    GPIO = 0x12         # I/O data
    OLAT = 0x14         # output latch

    # direct GPIO
    gpioPins = [12, 16, 18, 22, 15, 13, 11, 7]   # A/B
#            32, 36, 38, 40, 37, 35, 33, 31]     # B+

    def __init__(self, name, interface=None, event=None,
                                             addr=0x20,         # I2C address of MCP23017
                                             bank=0,            # bank within MCP23017 A=0, B=1
                                             inOut=0x00,        # I/O direction out=0, in=1
                                             interruptPin=17,   # RPIO pin used for interrupt (BCM number)
                                             config=[]):        # additional configuration
        Interface.__init__(self, name, interface=interface, event=event)
        global gpioInterfaces
        self.name = name
        if interface:
            self.addr = addr
            self.bank = bank
            self.inOut = inOut
            self.interruptPin = interruptPin+self.bank  # offset pin with bank
            self.config = config
            self.state = 0x00
            gpioInterfaces[self.interruptPin] = self
        else:
            self.interface = None
            self.bank = 0

    def start(self):
        debug('debugGPIO', self.name, "using GPIO library", gpioLibrary)
        gpio.setwarnings(False)
        if self.interface:
            gpio.setmode(gpio.BCM)
            # configure the MCP23017
            self.interface.write((self.addr, MCP23017Interface.IODIR+self.bank), self.inOut)    # I/O direction
            self.interface.write((self.addr, MCP23017Interface.GPINTEN+self.bank), self.inOut)  # enable interrupts for inputs
            self.interface.write((self.addr, MCP23017Interface.GPPU+self.bank), self.inOut)     # pull up resistors on inputs
            self.interface.write((self.addr, MCP23017Interface.IOCON), 0x04)                    # interrupt pins are open drain
            # additional configuration
            for config in self.config:
                reg = config[0]+self.bank   # offset register with bank
                debug('debugGPIO', self.name, "start", "addr: 0x%02x"%self.addr, "reg: 0x%02x"%reg, "value: 0x%02x"%config[1])
                self.interface.write((self.addr, reg), config[1])
            # get the current state
            self.readState()
            # set up the interrupt handling
            if gpioLibrary == "RPIO":
                gpio.add_interrupt_callback(self.interruptPin, interruptCallback, edge="falling", pull_up_down=gpio.PUD_UP)
                gpio.wait_for_interrupts(threaded=True)
            elif gpioLibrary == "RPi.GPIO":
                gpio.setup(self.interruptPin, gpio.IN, pull_up_down=gpio.PUD_UP)
                gpio.add_event_detect(self.interruptPin, gpio.FALLING, callback=interruptCallback)
        else:   # direct only supports output - FIXME
            gpio.setmode(gpio.BOARD)
            for pin in MCP23017Interface.gpioPins:
                debug('debugGPIO', self.name, "setup", pin, gpio.OUT)
                gpio.setup(pin, gpio.OUT)
                debug('debugGPIO', self.name, "write", pin, 0)
                gpio.output(pin, 0)

    # interrupt handler for this interface
    def interrupt(self):
        intFlags = self.interface.read((self.addr, MCP23017Interface.INTF+self.bank))
        debug('debugGPIO', self.name, "interrupt", "addr: 0x%02x"%self.addr, "bank:", self.bank, "intFlags: 0x%02x"%intFlags)
        self.readState()
        for i in range(8):
            if (intFlags >> i) & 0x01:
                try:
                    sensor = self.sensorAddrs[i]
                    state = (self.state >> i) & 0x01
                    debug('debugGPIO', self.name, "notifying", sensor.name, state)
                    sensor.notify(state)
                except KeyError:
                    debug('debugGPIO', self.name, "no sensor for interrupt on addr", i, self.sensorAddrs)


    def read(self, addr):
        if self.interface:
            self.readState()
            return (self.state >> addr) & 0x01
        else:
            return 0

    def readState(self):
        byte = self.interface.read((self.addr, MCP23017Interface.GPIO+self.bank))
        debug('debugGPIO', self.name, "read", "addr: 0x%02x"%self.addr, "reg: 0x%02x"%(MCP23017Interface.GPIO+self.bank), "value: 0x%02x"%byte)
        self.state = byte

    def write(self, addr, value):
        if self.interface:
            byte = self.state
            mask = 0x01<<addr
            byte = (byte & (~mask)) | ((value << addr) & mask)
            debug('debugGPIO', self.name, "write", "addr: 0x%02x"%self.addr, "reg: 0x%02x"%(MCP23017Interface.GPIO+self.bank), "value: 0x%02x"%byte)
            self.interface.write((self.addr, MCP23017Interface.GPIO+self.bank), byte)
            self.state = byte
        else:
            debug('debugGPIO', self.name, "write", "addr: 0x%02x"%addr, "value: 0x%02x"%value)
            gpio.output(MCP23017Interface.gpioPins[addr], value)



#  apt install swig
#  pip install rpi_ws281x
#  cd rpi_ws281x-master/python/
#  python ./setup.py install
#  mv neopixel.py ..

from neopixel import *
from homealone import *

stripRGB = 1050624
stripGRB = 528384

# convert a RGB tuple to a color
def color(r, g, b):
    return (r<<16 if r>0 else 0)+(g<<8 if g>0 else 0)+(b if b>0 else 0)

# convert a color to a RGB tuple
def uncolor(c):
    r = (c>>16) & 0xff
    g = (c>>8) & 0xff
    b = c & 0xff
    return (r, g, b)

# color definitions
all = color(255,255,255)
none = color(0,0,0)

white = color(255,191,127)
pink = color(255,63,63)
red = color(255,0,0)
orange = color(255,47,0)
yellow = color(255,127,0)
green = color(0,255,0)
blue = color(0,0,127)
purple = color(63,0,63)
indigo = color(31,0,47)
cyan = color(0,255,255)
magenta = color(255,0,63)
rust = color(63,7,0)

class NeopixelInterface(Interface):
    def __init__(self, name, interface, length=50, gpio=18, stripType=stripRGB, event=None):
        Interface.__init__(self, name, interface=interface, event=event)
        self.length = length
        self.gpio = gpio
        self.stripType = stripType

    def start(self):
        self.strip = Adafruit_NeoPixel(self.length, self.gpio, freq_hz=800000, dma=5, invert=False, strip_type=self.stripType)
        self.strip.begin()

    def read(self, addr):
        return None

    def write(self, addr, value):
        self.strip.setPixelColor(addr, value)

    def show(self):
        self.strip.show()
